/*
	LUSScreenSaver Plugin Interface
	Copyright (C) 2003 ONO,Masayuki.<mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSScreenSaverWidget.h"
#include "LUSScreenSaverMisc.h"
#include "LUSScreenSaver.h"

using namespace LUSMisc;

static const char* _version_ =
#include "./version.txt"
;
// popup-menu item id
static const int LUS_MI_SETTINGS = 1;
static const int LUS_MI_ACTIVATE = 2;
static const int LUS_MI_LCDOFF   = 3;
static const int LUS_MI_ABOUT    = 4;

LUSScreenSaverWidget::LUSScreenSaverWidget( LUSScreenSaver* saver, QWidget* parent )
	: QWidget(parent,NULL)
{
	m_pPopupMenu = NULL;
	m_pSaverImpl = saver;

	// m_oPixmap = Resource::loadPixmap( "LUSScreenSaver_icon" );
	// setFixedSize( m_oPixmap.size() );
	SetIconMode( GetIconMode() );

	m_pChannel = new QCopChannel( "QPE/LUSScreenSaver", this );
	connect( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );

	m_pSysChannel = new QCopChannel( "QPE/TaskBar", this );
	connect( m_pSysChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( sysMessage( const QCString&, const QByteArray& ) ) );
}

LUSScreenSaverWidget::~LUSScreenSaverWidget()
{
	disconnect ( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	delete m_pChannel;

	disconnect( m_pPopupMenu, SIGNAL( activated( int ) ),
							this, SLOT( MenuAction( int ) ) );
	delete m_pPopupMenu;

	delete m_pSysChannel;
}

void LUSScreenSaverWidget::receiveMessage( const QCString& msg, const QByteArray& data )
{
	QDataStream stream(data, IO_ReadOnly);
	if ( msg == "SetScreenSaverActivate()" )
	{
		SetScreenSaverActivate();
	} 
	else if ( msg == "SetLCDBlank()")
	{
		SetLCDBlank();
	}
	else if ( msg == "ShowIcon(int)")
	{
		int mode;
		stream >> mode;
		SetIconMode( mode );
		repaint();
	}
	else if ( msg == "enable()")
	{
		;
	}
	else if ( msg == "disable()")
	{
		;
	}
	else if ( msg == "Version()")
	{
		Version();
	}
}

void LUSScreenSaverWidget::SetIconMode( int mode )
{
	if ( mode == 1 )
	{
		m_oPixmap = Resource::loadPixmap( "LUSScreenSaver_icon" );
	}
	else
	{
		m_oPixmap.resize( 0, 0 );
	}

	setFixedSize( m_oPixmap.size() );
	SaveIconMode( mode );
}

void LUSScreenSaverWidget::sysMessage( const QCString& msg, const QByteArray& data )
{
	QDataStream stream(data, IO_ReadOnly);
	if ( msg == "reloadApplets()" )
	{
		// QWSServer::setScreenSaver() operation must cause reboot.
		if ( m_pSaverImpl )
		{
			m_pSaverImpl->ExecOKDlg( tr("You come across Reboot") );
		}
	}
}

void LUSScreenSaverWidget::Version()
{
	QString ver = AppName();
	ver.append( _version_ );
	Global::statusMessage( ver );
}

void LUSScreenSaverWidget::SetScreenSaverActivate()
{
	if ( m_pSaverImpl )
	{
		m_pSaverImpl->SetForceExecFlag();
		QWSServer::setScreenSaverInterval( 800 );
	}
}

void LUSScreenSaverWidget::mouseReleaseEvent( QMouseEvent * )
{
	if ( !m_pPopupMenu )
	{
	m_pPopupMenu = new QPopupMenu( this );

	m_pPopupMenu->insertItem( tr( " Settings " ), LUS_MI_SETTINGS );
	m_pPopupMenu->insertItem( tr( " Activate " ), LUS_MI_ACTIVATE );
	m_pPopupMenu->insertItem( tr( " LCD Off  " ), LUS_MI_LCDOFF   );
	m_pPopupMenu->insertSeparator();
	m_pPopupMenu->insertItem( tr( " about .. " ), LUS_MI_ABOUT    );

	connect( m_pPopupMenu, SIGNAL( activated( int ) ),
								this, SLOT( MenuAction( int ) ) );
	}

	QPoint oPos = mapToGlobal( QPoint( 0, 0 ) );
	QSize sizeMenu = m_pPopupMenu->sizeHint();
	m_pPopupMenu->popup( oPos - QPoint( ( sizeMenu.width() - width() ) / 2,
													sizeMenu.height() ) );
}

void LUSScreenSaverWidget::MenuAction( int nId )
{
	switch ( nId )
	{
	case LUS_MI_SETTINGS:
		ExecSettingsDialog();
		break;
	case LUS_MI_ACTIVATE:
		SetScreenSaverActivate();
		break;
	case LUS_MI_LCDOFF:
		SetLCDBlank();
		break;
	case LUS_MI_ABOUT:
		m_pSaverImpl->ExecOKDlg( QString(
											"Less UnSaticefactory\n"\
											" ScreenSaver Ver. %1\n"\
											" (C) 2003  M.Ono\n"\
											"            GPL Lisenced"\
										 )
										.arg( _version_ ).latin1() );
		break;
	default:
		break;
	}
	delete m_pPopupMenu;
	m_pPopupMenu = NULL;
}

void LUSScreenSaverWidget::ExecSettingsDialog()
{
	Global::execute( "LUSScreenSaverUtil" );
}

void LUSScreenSaverWidget::SetLCDBlank()
{
	QCopEnvelope e( "QPE/System", "setBlankLCD(int)" );
	e << 1;
}

void LUSScreenSaverWidget::paintEvent(QPaintEvent *)
{
	QPainter p(this);
	p.drawPixmap(0, 0, m_oPixmap);
}
